# -*- coding: utf-8 -*-
"""  frankhood.fhcore.views.view_mixins.py
    @author: Frankhood Business Solutions
    @date : 20/set/2013
"""
import csv
import json
import os
import  logging
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from django.core.serializers.json import DjangoJSONEncoder
from django.http import HttpResponse
from django.views.generic.base import TemplateResponseMixin
from django.views.generic.list import BaseListView
from django.views.static import serve

__all__ = [
    'TXTResponseMixin',
    'PDFResponseMixin',
    'CSVResponseMixin',
    'FileResponseServeMixin',
    'ImageResponseServeMixin',
]

logger = logging.getLogger(__name__)

from response_view_mixin.utils import image_serve


# ===============================================================================
# TXT
# ===============================================================================
class TXTResponseMixin(object):
    file_name = None
    file_mimetype = 'text/plain'

    def get_file_name(self):
        return self.file_name

    def get_file_mimetype(self):
        return self.file_mimetype

    def build_response(self, response, context):
        for row in context['rows']:
            response.write("%s\n" % row)
        return response

    def render_to_response(self, context, **response_kwargs):
        """
        Genera una risposta nel mimetype selezionato.
        """
        file_name = self.get_file_name()
        file_mimetype = self.get_file_mimetype()
        if not file_name:
            raise Exception("Manca il nome del file")
        response = HttpResponse(content_type=file_mimetype)
        response['Content-Disposition'] = 'attachment; filename="%s"' % file_name
        response = self.build_response(response, context)
        return response


# ===============================================================================
# PDF
# ===============================================================================
class PDFResponseMixin(object):
    file_name = None
    file_mimetype = "application/pdf"

    def get_file_name(self):
        return self.file_name

    def get_file_dir(self):
        # modifica se non è questo il path del file
        return os.path.join(settings.STATIC_ROOT, 'uploads', 'pdf')

    def render_to_response(self, context, **response_kwargs):
        file_path = "{0}{1}{2}".format(self.get_file_dir(), os.sep, self.get_file_name())
        with open(file_path, 'rb') as pdf:
            response = HttpResponse(pdf.read(), content_type=self.file_mimetype)
            response['Content-Disposition'] = 'inline;filename={0}'.format(self.get_file_name())
            return response


# ===============================================================================
# CSV
# ===============================================================================
class CSVResponseMixin(TXTResponseMixin):
    csv_delimiter = ";"
    file_mimetype = 'text/csv'

    def build_response(self, response, context):
        writer = csv.writer(response, delimiter=self.csv_delimiter)
        for row in context['rows']:
            writer.writerow([s.encode("utf-8") if isinstance(s, str) else s for s in row])
        return response


# ============================================================================
# FILES
# ============================================================================
class FileResponseServeMixin(object):
    use_private_storage = False

    """ Mixin che fa restituire un file invece di una HttpResponse """

    def get_file_path(self, **kwargs):
        raise NotImplementedError()

    def get_file_docroot(self, **kwargs):
        if self.use_private_storage:
            if not hasattr(settings, "PRIVATE_ROOT"):
                logger.error("Cannot use PRIVATE_FILE_SYSTEM without settings PRIVATE_ROOT")
                return settings.PROJECT_ROOT
            return settings.PRIVATE_ROOT
        else:
            return settings.PROJECT_ROOT

    def render_to_response(self, context, **response_kwargs):
        context = context or {}
        response = serve(self.request,
                         path=self.get_file_path(**context),
                         document_root=self.get_file_docroot())
        try:
            response['Content-Disposition'] = ('attachment; filename="{0}"'
                                               '').format(response.get('Content-Disposition',
                                                                       os.path.split(self.get_file_path(**context))[
                                                                           -1]))
        except Exception:
            logger.exception("")
        return response


class ImageResponseServeMixin(FileResponseServeMixin):
    """ Mixin che fa restituire un'immagine invece di una HttpResponse"""

    def render_to_response(self, context, **response_kwargs):
        context = context or {}
        # X l'immagine nn dobiamo scaricarla, quindi nn dobbiamo modificare l'header
        # scrivendo response['Content-Disposition'] = 'attachment; {0}'.format(response['Content-Disposition'])
        return image_serve(self.request,
                           path=self.get_file_path(**context),
                           document_root=self.get_file_docroot())



# -*- coding: utf-8 -*-
""" fhcore.utils.image_serve
    @author: Frankhood Business Solutions
    @created: 04 apr 2018
"""
from __future__ import absolute_import, print_function, unicode_literals
import logging
import mimetypes
import os
import posixpath
import stat

from django.utils.translation import ugettext, ugettext_lazy as _
from django.conf import settings
from django.http import (StreamingHttpResponse, Http404,
                         HttpResponse, HttpResponseRedirect, HttpResponseNotModified)
from django.utils.http import http_date, parse_http_date
from django.views.static import was_modified_since, directory_index

try:
    from urllib.parse import unquote
except ImportError:  # Python 2
    from urllib import unquote

logger = logging.getLogger(__name__)


def image_serve(request, path, document_root=None, show_indexes=False):
    """
    Reimplementation of django.views.static.serve function
    The difference with the django.views.static.serve function is that
    it uses mimetype image/jpeg as default and not octet/stream...
    """
    path = posixpath.normpath(unquote(path))
    path = path.lstrip('/')
    newpath = ''
    for part in path.split('/'):
        if not part:
            # Strip empty path components.
            continue
        _drive, part = os.path.splitdrive(part)
        _head, part = os.path.split(part)
        if part in (os.curdir, os.pardir):
            # Strip '.' and '..' in path.
            continue
        newpath = os.path.join(newpath, part).replace('\\', '/')
    if newpath and path != newpath:
        return HttpResponseRedirect(newpath)
    fullpath = os.path.join(document_root, newpath)
    if os.path.isdir(fullpath):
        if show_indexes:
            return directory_index(newpath, fullpath)
        raise Http404(_("Directory indexes are not allowed here."))
    if not os.path.exists(fullpath):
        raise Http404(_('"%(path)s" does not exist') % {'path': fullpath})
    # Respect the If-Modified-Since header.
    statobj = os.stat(fullpath)
    mimetype, encoding = mimetypes.guess_type(fullpath)
    mimetype = mimetype or 'image/jpeg'
    if not was_modified_since(request.META.get('HTTP_IF_MODIFIED_SINCE'),
                              statobj.st_mtime, statobj.st_size):
        return HttpResponseNotModified()
    response = StreamingHttpResponse(open(fullpath, 'rb'), content_type=mimetype)
    response["Last-Modified"] = http_date(statobj.st_mtime)
    if stat.S_ISREG(statobj.st_mode):
        response["Content-Length"] = statobj.st_size
    if encoding:
        response["Content-Encoding"] = encoding
    return response
